#if !defined( DXX_VERTEXBUFFERPROXY_INL_INCLUDED )
#define DXX_VERTEXBUFFERPROXY_INL_INCLUDED

#pragma once

/** @file *//********************************************************************************************************

                                                 VertexBufferProxy.inl

	--------------------------------------------------------------------------------------------------------------

	$Header: //depot/Libraries/Dxx/VertexBufferProxy.inl#2 $

	$NoKeywords: $

*********************************************************************************************************************/

#include "VertexBufferProxy.h"

namespace Dxx
{

/********************************************************************************************************************/
/********************************************************************************************************************/
/*																													*/
/*										V E R T E X B U F F E R P R O X Y											*/
/*																													*/
/********************************************************************************************************************/
/********************************************************************************************************************/

inline VertexBufferProxy::~VertexBufferProxy()
{
	m_pBuffer->Release();
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline IDirect3DVertexBuffer9 * VertexBufferProxy::GetVertexBuffer() const
{
	return m_pBuffer;
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

/// @param	OffsetToLock	Offset to start of locked region
/// @param	SizeToLock		Number of bytes to lock
/// @param	Flags			Flags (see docs for D3DLOCK)

inline HRESULT VertexBufferProxy::Lock( UINT OffsetToLock, UINT SizeToLock, DWORD Flags )
{
	m_LockOffset = OffsetToLock;
	return m_pBuffer->Lock( OffsetToLock, SizeToLock, (void **)&m_pData, Flags );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline HRESULT VertexBufferProxy::Unlock()
{
	m_pData			= 0;
	m_LockOffset	= 0;
	return m_pBuffer->Unlock();
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

///
/// @param	i	Index of vertex to point to (relative to locked region)

inline void * VertexBufferProxy::GetVertex( int i ) const
{
	return (void *)( (char *)m_pData + ( m_Stride * i - m_LockOffset ) );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

///
/// @param	i	Index of vertex to get (relative to locked region)

inline VertexBufferProxy::VRef VertexBufferProxy::operator []( int i ) const
{
	return VRef( this, GetVertex( i ) );
}


/********************************************************************************************************************/
/********************************************************************************************************************/
/*																													*/
/*												V R E F B A S E														*/
/*																													*/
/********************************************************************************************************************/
/********************************************************************************************************************/

inline VertexBufferProxy::VRefBase::VRefBase( VertexBufferProxy const * pProxy, void * pVertex )
	: m_pProxy( pProxy ), m_pVertex( pVertex )
{
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline D3DXVECTOR3 & VertexBufferProxy::VRefBase::Position() const
{
	return *(D3DXVECTOR3 *)( (char *)m_pVertex + 0 );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline float & VertexBufferProxy::VRefBase::W() const
{
	return *(float *)( (char *)m_pVertex + m_pProxy->m_WOffset );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline float & VertexBufferProxy::VRefBase::Blend0() const
{
	return *(float *)( (char *)m_pVertex + m_pProxy->m_Blend0Offset );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline float & VertexBufferProxy::VRefBase::Blend1() const
{
	return *(float *)( (char *)m_pVertex + m_pProxy->m_Blend1Offset );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline float & VertexBufferProxy::VRefBase::Blend2() const
{
	return *(float *)( (char *)m_pVertex + m_pProxy->m_Blend2Offset );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline float & VertexBufferProxy::VRefBase::Blend3() const
{
	return *(float *)( (char *)m_pVertex + m_pProxy->m_Blend3Offset );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline float & VertexBufferProxy::VRefBase::BlendIndexes() const
{
	return *(float *)( (char *)m_pVertex + m_pProxy->m_BlendIndexesOffset );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline D3DXVECTOR3 & VertexBufferProxy::VRefBase::Normal() const
{
	return *(D3DXVECTOR3 *)( (char *)m_pVertex + m_pProxy->m_NormalOffset );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline float & VertexBufferProxy::VRefBase::PointSize() const
{
	return *(float *)( (char *)m_pVertex + m_pProxy->m_PointSizeOffset );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline DWORD & VertexBufferProxy::VRefBase::Diffuse() const
{
	return *(DWORD *)( (char *)m_pVertex + m_pProxy->m_DiffuseOffset );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline DWORD & VertexBufferProxy::VRefBase::Specular() const
{
	return *(DWORD *)( (char *)m_pVertex + m_pProxy->m_SpecularOffset );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline float * VertexBufferProxy::VRefBase::TexCoord0() const
{
	return (float *)( (char *)m_pVertex + m_pProxy->m_TexCoord0Offset );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline float * VertexBufferProxy::VRefBase::TexCoord1() const
{
	return (float *)( (char *)m_pVertex + m_pProxy->m_TexCoord1Offset );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline float * VertexBufferProxy::VRefBase::TexCoord2() const
{
	return (float *)( (char *)m_pVertex + m_pProxy->m_TexCoord2Offset );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline float * VertexBufferProxy::VRefBase::TexCoord3() const
{
	return (float *)( (char *)m_pVertex + m_pProxy->m_TexCoord3Offset );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline float * VertexBufferProxy::VRefBase::TexCoord4() const
{
	return (float *)( (char *)m_pVertex + m_pProxy->m_TexCoord4Offset );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline float * VertexBufferProxy::VRefBase::TexCoord5() const
{
	return (float *)( (char *)m_pVertex + m_pProxy->m_TexCoord5Offset );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline float * VertexBufferProxy::VRefBase::TexCoord6() const
{
	return (float *)( (char *)m_pVertex + m_pProxy->m_TexCoord6Offset );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline float * VertexBufferProxy::VRefBase::TexCoord7() const
{
	return (float *)( (char *)m_pVertex + m_pProxy->m_TexCoord7Offset );
}


/********************************************************************************************************************/
/********************************************************************************************************************/
/*																													*/
/*													V P T R															*/
/*																													*/
/********************************************************************************************************************/
/********************************************************************************************************************/

/// @param	pProxy		VertexBufferProxy this is associated with
/// @param	pVertex		Vertex this refers to

inline VertexBufferProxy::VPtr::VPtr( VertexBufferProxy const * pProxy, void * pVertex )
		: VRefBase( pProxy, pVertex )
{
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline VertexBufferProxy::VRef VertexBufferProxy::VPtr::operator*() const
{
	return VertexBufferProxy::VRef( m_pProxy, m_pVertex );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

///
/// @param	i	index relative to this vertex

inline VertexBufferProxy::VRef VertexBufferProxy::VPtr::operator[]( int i ) const
{
	return *( *this + i );
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline VertexBufferProxy::VRefBase const * VertexBufferProxy::VPtr::operator->() const
{
	return this;
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline VertexBufferProxy::VRefBase * VertexBufferProxy::VPtr::operator->()
{
	return this;
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline VertexBufferProxy::VPtr & VertexBufferProxy::VPtr::operator++()
{
	m_pVertex = (char *)m_pVertex + m_pProxy->m_Stride;
	return *this;
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline VertexBufferProxy::VPtr VertexBufferProxy::VPtr::operator++( int )
{
	VPtr	v( *this );
	m_pVertex = (char *)m_pVertex + m_pProxy->m_Stride;
	return v;
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline VertexBufferProxy::VPtr & VertexBufferProxy::VPtr::operator--()
{
	m_pVertex = (char *)m_pVertex - m_pProxy->m_Stride;
	return *this;
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline VertexBufferProxy::VPtr VertexBufferProxy::VPtr::operator--( int )
{
	VPtr	v( *this );
	m_pVertex = (char *)m_pVertex - m_pProxy->m_Stride;
	return v;
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

///
/// @param	i	Value to add

inline VertexBufferProxy::VPtr & VertexBufferProxy::VPtr::operator+=( int i )
{
	m_pVertex = (char *)m_pVertex + ( i * m_pProxy->m_Stride );
	return *this;
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

///
/// @param	i	Value to subtract

inline VertexBufferProxy::VPtr & VertexBufferProxy::VPtr::operator-=( int i )
{
	m_pVertex = (char *)m_pVertex - ( i * m_pProxy->m_Stride );
	return *this;
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline VertexBufferProxy::VPtr::operator void*() const
{
	return m_pVertex;
}


} // namespace Dxx


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

/// @param	vptr	Operand
/// @param	i		Operand

inline Dxx::VertexBufferProxy::VPtr operator+( Dxx::VertexBufferProxy::VPtr const & vptr, int i )
{
	return Dxx::VertexBufferProxy::VPtr( vptr ) += i;
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

/// @param	vptr	Operand
/// @param	i		Operand

inline Dxx::VertexBufferProxy::VPtr operator-( Dxx::VertexBufferProxy::VPtr const & vptr, int i )
{
	return Dxx::VertexBufferProxy::VPtr( vptr ) -= i;
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

/// @param	i		Operand
/// @param	vptr	Operand

inline Dxx::VertexBufferProxy::VPtr operator+( int i, Dxx::VertexBufferProxy::VPtr const & vptr )
{
	return operator+( vptr, i );
}


/********************************************************************************************************************/
/********************************************************************************************************************/
/*																													*/
/*													V R E F															*/
/*																													*/
/********************************************************************************************************************/
/********************************************************************************************************************/


namespace Dxx
{

/// @param	pProxy		VertexBufferProxy this is associated with
/// @param	pVertex		Vertex this refers to

inline VertexBufferProxy::VRef::VRef( VertexBufferProxy const * pProxy, void * pVertex )
	: VRefBase( pProxy, pVertex )

{
}


/********************************************************************************************************************/
/*																													*/
/*																													*/
/********************************************************************************************************************/

inline VertexBufferProxy::VPtr VertexBufferProxy::VRef::operator &()
{
	return VPtr( m_pProxy, m_pVertex );
}


} // namespace Dxx


#endif // !defined( DXX_VERTEXBUFFERPROXY_INL_INCLUDED )
